"""
Monte Carlo Markov Chain
------------------------

Create of pure python MCMC runs using the BORG library.

"""

import numpy as np
import aquila_borg as borg
from typing import Tuple
from dataclasses import dataclass


@dataclass
class MarkovRun:
    loop: borg.samplers.MainLoop
    info: borg.likelihood.LikelihoodInfo
    Ngs: tuple = (128, 128, 128)

    def addCatalog(self):
        state = self.state
        cat = int(state["NCAT"][:])
        state["NCAT"][:] += 1
        state.newArray1d(f"galaxy_bias_{cat}", 1)
        state.newScalar(f"galaxy_nmean_{cat}", 1.0)
        state.newScalar(f"galaxy_bias_ref_{cat}", False)
        state.newArray3d(f"galaxy_sel_window_{cat}", *self.Ngs)
        state.newArray3d(f"galaxy_synthetic_sel_window_{cat}", *self.Ngs)
        state.newArray3d(f"galaxy_data_{cat}", *self.Ngs)

        state[f"galaxy_sel_window_{cat}"][:] = 1.0
        state[f"galaxy_data_{cat}"][:] = 0.0

    @property
    def state(self) -> borg.likelihood.MarkovState:
        return self.loop.getState(0)

    @property
    def cosmology(self) -> borg.cosmo.CosmologicalParameters:
        return self.state["cosmology"]


def setupMarkovLoop(
    box: borg.forward.BoxModel,
    catalog_N: Tuple[int, int, int],
    chain: borg.forward.BORGForwardModel,
):

    # Create an info object
    info = borg.likelihood.LikelihoodInfo()

    # Fill up with some grid length
    info["GRID_LENGTH"] = np.array(
        list(np.array([box.xmin[j], box.xmin[j] + box.L[j]]) for j in range(3))
    ).reshape((6,))
    info["GRID"] = np.array(box.N, dtype=np.uint64)

    # Create a main loop of samplers with the attached MarkovStates
    loop = borg.samplers.MainLoop()
    # Ask for setting a rand gen
    loop.addRandomGenerator()

    # Get the first MCMC
    state = loop.getState(0)

    # Push a cosmology object
    state.newScalar("cosmology", borg.cosmo.CosmologicalParameters())

    # Push a forward model
    state.newForwardModel("BORG_model", chain)

    state.newScalar("ares_heat", 1.0)
    state.newScalar("hades_sampler_blocked", False)
    state.newScalar("MCMC_STEP", 0, type_code="L")

    # Setup fake catalogs
    state.newScalar("NCAT", 0, type_code="L")

    # Copy again the grid spec
    for i in range(3):
        state.newScalar(f"N{i}", box.N[i], type_code="L")
        state.newScalar(f"L{i}", box.L[i])
        state.newScalar(f"corner{i}", box.xmin[i])
        state.newScalar(f"localNdata{2*i}", 0, type_code="L")
        state.newScalar(f"localNdata{2*i+1}", catalog_N[i], type_code="L")

    return MarkovRun(info=info, loop=loop)
